@@ -48,7 +48,7 @@ class APIDefinition(object):
 
 class ExtraActionDefinition(object):
     """Definition of a "non-standard" function"""
-    url = None
+    url = ''
     url_params = []
     required_params = []
     optional_params = []
@@ -84,7 +84,7 @@ class Group(APIDefinition):
 
     class TransferProjectAction(ExtraActionDefinition):
         """gl.Group.transfer_project(project_id)"""
-        url = '/groups/:id/projects/:project_id'
+        url = '/projects/:project_id'
         method = _HTTP_POST
         url_params = [
             'project_id',
@@ -101,7 +101,6 @@ class SystemHook(APIDefinition):
 
     class TestAction(ExtraActionDefinition):
         """gl.Hook.test()"""
-        url = '/hooks/:id'
         method = _HTTP_GET
     extra_actions = [ TestAction ]
 
@@ -243,6 +242,22 @@ class Project(APIDefinition):
             'state_event',
         ]
         sub_apis = [ Note ]
+        class CloseAction(ExtraActionDefinition):
+            """gl.Project.Issue.close()"""
+            method = _HTTP_PUT
+            required_params = [ 'state_event' ]
+            _state_after = 'closed'
+            @classmethod
+            def wrapper(cls, extra_action_fn, parent):
+                def wrapped(self):
+                    extra_action_fn(self, state_event=cls.name())
+                    setattr(self, 'state', cls._state_after)
+                return wrapped
+        class ReopenAction(CloseAction):
+            _state_after = 'reopened'
+
+        extra_actions = [ CloseAction, ReopenAction ]
+
 
     class Branch(APIDefinition):
         url = '/repository/branches/:branch'
@@ -291,7 +306,7 @@ class Project(APIDefinition):
 
         class PostCommentAction(ExtraActionDefinition):
             """gl.Project.MergeRequest.post_comment(note)"""
-            url = '/merge_request/:merge_request_id/comments'
+            url = '/comments'
             method = _HTTP_POST
             required_args = [
                 'note',
@@ -324,12 +339,14 @@ class Project(APIDefinition):
             'lifetime',
         ]
 
-        class GetRawAction(ExtraActionDefinition):
-            """gl.Project.Snippet.get_raw()"""
-            url = '/snippets/:snippet_id/raw'
+        class RawAction(ExtraActionDefinition):
+            """gl.Project.Snippet.raw()"""
+            url = '/raw'
             method = _HTTP_GET
+        class GetRawAction(RawAction):
+            pass
 
-        extra_actions = [ GetRawAction ]
+        extra_actions = [ RawAction, GetRawAction ]
         sub_apis = [ Note ]
 
     class Tag(APIDefinition):
@@ -346,11 +363,18 @@ class Project(APIDefinition):
         ]
 
     class Commit(APIDefinition):
-        url = '/repository/commits'
-        actions = [ _LIST ]
+        url = '/repository/commits/:sha'
+        actions = [ _LIST, _GET ]
         optional_params = [
             'ref_name',
         ]
+        class DiffAction(ExtraActionDefinition):
+            """gl.Project.Commit.diff()"""
+            url = '/diff'
+            method = _HTTP_GET
+        class GetDiffAction(DiffAction):
+            pass
+        extra_actions = [ DiffAction, GetDiffAction ]
 
     class WallNote(Note):
         @classmethod
@@ -400,6 +424,7 @@ class User(APIDefinition):
 
 
 class Team(APIDefinition):
+    url = '/user_teams/:id'
     actions =  [ _LIST, _GET, _ADD ]
     required_params = [
         'name',
@@ -438,7 +463,6 @@ class GitLab(APIDefinition):
             'snippets_enabled',
             'public',
         ]
-
         @staticmethod
         def wrapper(extra_action_fn, parent):
             def wrapped(*args, **kwargs):
